<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
xmlns:fx="http://www.asam.net/xml/fbx" 
xmlns:ho="http://www.asam.net/xml" 
xmlns:n1="http://www.evidence.eu/schemas/rtdruid" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
<xsl:output method="xml" indent="yes" />

    <xsl:template match="/">
          <xsl:element name="SYSTEM">
		<xsl:attribute name="Name"><xsl:value-of select="fx:FIBEX/fx:PROJECT/@ID"/></xsl:attribute>          
	      <xsl:apply-templates />
	    </xsl:element>             
    </xsl:template>
    
    <xsl:template match="fx:FUNCTIONS"> 
          <xsl:element name="FUNCTIONAL">
	      <xsl:apply-templates />
	      <xsl:call-template name="varstemplate"/>
	    </xsl:element>       
    </xsl:template>
    
    <xsl:template match="fx:ELEMENTS">
	      <xsl:apply-templates />
    </xsl:template>

    <xsl:template match="fx:PROJECT"></xsl:template>
    <xsl:template match="fx:SIGNALS"></xsl:template>
    
    <xsl:template match="fx:REQUIREMENTS">
          <xsl:element name="ANNOTATION">
			  <xsl:for-each select="fx:FUNCTION-REQS/fx:FUNCTION-REQ/fx:W-C-E-T" >
				  <xsl:element name="EXECTIME">
					  <xsl:attribute name="Ref">
						  <xsl:variable name="funct" select="../fx:FUNCTION-REFS/fx:FUNCTION-REF/@ID-REF"/>
						  <xsl:value-of select="../../../../fx:ELEMENTS/fx:FUNCTIONS/fx:FUNCTION[@ID=$funct]/ho:SHORT-NAME"/></xsl:attribute>
	      <xsl:attribute name="Type">proc</xsl:attribute>
	      <xsl:element name="WORST">
	      <xsl:attribute name="Value">
	      <xsl:value-of select="substring-before(substring(fx:VALUE, 3), 'S')"/>
	      </xsl:attribute>
	      </xsl:element>
	      </xsl:element>
	      </xsl:for-each>
	    </xsl:element>       
    </xsl:template>

    <xsl:template match="fx:PROCESSING-INFORMATION"></xsl:template>
    
    <xsl:template match="fx:FUNCTION"> 
         <xsl:element name="PROC"><xsl:attribute name="Name"><xsl:value-of select="ho:SHORT-NAME"/></xsl:attribute>          
         <xsl:for-each select="fx:INPUT-PORTS/fx:INPUT-PORT">
         <xsl:element name="METHOD">
         <xsl:attribute name="Name">
         <xsl:variable name="nodeid" select="fx:SIGNAL-REF/@ID-REF"/>
         <xsl:value-of select="../../../../fx:SIGNALS/fx:SIGNAL[@ID=$nodeid]/ho:SHORT-NAME"/>.read</xsl:attribute>          
         </xsl:element>       
         </xsl:for-each>          
         <xsl:for-each select="fx:OUTPUT-PORTS/fx:OUTPUT-PORT">
         <xsl:element name="METHOD">
         <xsl:attribute name="Name">
         <xsl:variable name="nodeid" select="fx:SIGNAL-REF/@ID-REF"/>
         <xsl:value-of select="../../../../fx:SIGNALS/fx:SIGNAL[@ID=$nodeid]/ho:SHORT-NAME"/>.write</xsl:attribute>
         </xsl:element>          	   
         </xsl:for-each>          
	   </xsl:element>   
	          
	   <xsl:element name="EVENT">
		   <xsl:attribute name="Name"><xsl:value-of select="ho:SHORT-NAME"/>@activation</xsl:attribute>          
		   <xsl:attribute name="Type">activation</xsl:attribute>          
		   <xsl:attribute name="MethodRefName"><xsl:value-of select="ho:SHORT-NAME"/>.run</xsl:attribute>          
	   </xsl:element> 
	   <xsl:element name="EVENT">
		   <xsl:attribute name="Name"><xsl:value-of select="ho:SHORT-NAME"/>@begin</xsl:attribute>          
		   <xsl:attribute name="Type">begin</xsl:attribute>          
		   <xsl:attribute name="MethodRefName"><xsl:value-of select="ho:SHORT-NAME"/>.run</xsl:attribute>          
	   </xsl:element> 
	   <xsl:element name="EVENT">
		   <xsl:attribute name="Name"><xsl:value-of select="ho:SHORT-NAME"/>@end</xsl:attribute>          
		   <xsl:attribute name="Type">end</xsl:attribute>          
		   <xsl:attribute name="MethodRefName"><xsl:value-of select="ho:SHORT-NAME"/>.run</xsl:attribute>          
	   </xsl:element> 

        <xsl:variable name="testt" select="@ID"/>	   
        <xsl:variable name="fname" select="ho:SHORT-NAME"/>	   
	   <xsl:for-each select="../../../fx:REQUIREMENTS/fx:FUNCTION-REQS/fx:FUNCTION-REQ">
	   <xsl:if test="fx:FUNCTION-REFS/fx:FUNCTION-REF/@ID-REF=$testt">
		   <xsl:if test="fx:CYCLE-PERIOD/fx:VALUE">
			   <xsl:element name="TIMECONST">
			   <xsl:element name="EVENTREF"><xsl:attribute name="Name"><xsl:value-of select="$fname"/>@activation</xsl:attribute>
			   </xsl:element>
			   <xsl:element name="BOUND">
				  <xsl:attribute name="Type">period</xsl:attribute>
				  <xsl:attribute name="Value"><xsl:value-of select="substring-before(substring(fx:CYCLE-PERIOD/fx:VALUE, 3), 'S')"/></xsl:attribute>
			   </xsl:element>
			   </xsl:element>
		   </xsl:if>
	   
			<xsl:if test="fx:DEADLINE/fx:VALUE">
			   <xsl:element name="TIMECONST">
			   <xsl:element name="EVENTREF"><xsl:attribute name="Name"><xsl:value-of select="$fname"/>@activation</xsl:attribute>
			   </xsl:element>
			   <xsl:element name="EVENTREF"><xsl:attribute name="Name"><xsl:value-of select="$fname"/>@end</xsl:attribute>
			   </xsl:element>
			   <xsl:element name="BOUND">
				  <xsl:attribute name="Type">deadline</xsl:attribute>
				  <xsl:attribute name="Value"><xsl:value-of select="substring-before(substring(fx:DEADLINE/fx:VALUE, 3), 'S')"/></xsl:attribute>
			   </xsl:element>
			   </xsl:element>
		   </xsl:if>

	   </xsl:if>
	   </xsl:for-each>
    </xsl:template>
    
 <xsl:template name="varstemplate">
	      <xsl:for-each select="../fx:SIGNALS/fx:SIGNAL">
<xsl:element name="VAR"><xsl:attribute name="Name">
<xsl:value-of select="ho:SHORT-NAME"/></xsl:attribute>
<xsl:attribute name="Type">
<xsl:variable name="vartype" select="fx:CODING-REF/@ID-REF"/>
<xsl:value-of select="../../../fx:PROCESSING-INFORMATION/fx:CODINGS/fx:CODING[@ID=$vartype]/ho:CODED-TYPE/@ho:BASE-DATA-TYPE"/>
</xsl:attribute>          
	   </xsl:element>          	   
         </xsl:for-each>          
    </xsl:template>
        <xsl:template match="*" priority="-1">
	      <xsl:apply-templates />
    </xsl:template>

</xsl:stylesheet>
